/*
 modified 21 July 2015
 by Mauro Batini
 */

#include <SPI.h>
#include <Ethernet.h>
#include <AccelStepper.h>


String inputString = ""; // a string to hold incoming data
boolean stringComplete = false; // whether the string is complete                                     
double turnHeading, turnPitch, turnRoll;
double prevTurnPitch = 0.0, prevTurnRoll = 0.0, prevTurnHeading = 0.0;
char c;
String heading,pitch,roll;

// Enter a MAC address and IP address for your controller below.
// The IP address will be dependent on your local network:
byte mac[] = {  
  0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0x1B };
IPAddress ip(10,10,70,180);

// Enter the IP address of the server you're connecting to:
IPAddress server(10,10,70,137); 

// Initialize the Ethernet client library
// with the IP address and port of the server that you want to connect to
EthernetClient client;

// Define a stepper and the pins it will use
AccelStepper stepperX(1, 2, 5); // pin 2 = step, pin 5 = direction ROLL
AccelStepper stepperY(1, 3, 6); // pin 3 = step, pin 6 = direction YAW
AccelStepper stepperZ(1, 4, 7); // pin 4 = step, pin 7 = direction PITCH

void setup() {
  pinMode(8, OUTPUT); //enable LOW
  digitalWrite(8, LOW);

  stepperX.setMaxSpeed(400);
  stepperX.setAcceleration(400);
  stepperX.setSpeed(300);

  stepperY.setMaxSpeed(400);
  stepperY.setAcceleration(400);
  stepperY.setSpeed(300);

  stepperZ.setMaxSpeed(400);
  stepperZ.setAcceleration(400);
  stepperZ.setSpeed(300);

  // start the Ethernet connection:
  Ethernet.begin(mac, ip);

  // Open serial communications and wait for port to open:
  Serial.begin(115200);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for Leonardo only
  }

  // give the Ethernet shield a second to initialize:
  delay(1000);
  Serial.println("connecting...");

  // if you get a connection, report back via serial:
  if (client.connect(server, 8000)) {
    Serial.println("connected");
  } 
  else {
    // if you didn't get a connection to the server:
    Serial.println("connection failed");
  }
}

void loop()
{
  stepperX.run();
  stepperY.run();
  stepperZ.run();
  // if there are incoming bytes available 
  // from the server, read them and print them:

  if (client) {
    if (client.connected()) {
      while (client.available()) {
        c = client.read();
        // if we have a complete string, calculate values and move motors
        if( c == '\n') {
          //load h,p,r with substrings
          //Serial.println("stringa"+inputString);
          parseInput();
          //Serial.println("dopo"+heading + pitch + roll);
          if((stepperX.distanceToGo() ==0) && ((abs(turnRoll - prevTurnRoll) > 1))) {
            stepperX.move(turnRoll - prevTurnRoll);
            prevTurnRoll = turnRoll;
            Serial.print("R \t");

            Serial.println(turnRoll);
          }
          if((stepperY.distanceToGo() ==0) && ((abs(turnHeading - prevTurnHeading) > 1))) {
            stepperY.move(turnHeading - prevTurnHeading);
            prevTurnHeading = turnHeading;
            Serial.print("H \t");
            Serial.println(turnHeading);
          }
          if((stepperZ.distanceToGo() ==0) && ((abs(turnPitch - prevTurnPitch) > 1))) {
            stepperZ.move(turnPitch - prevTurnPitch);
            prevTurnPitch = turnPitch;
            Serial.print("P \t");
            Serial.println(turnPitch);
          }
          inputString = "";
        } 
        else {
          inputString += c; 
        }
      }
    }
  }

  // if the server's disconnected, stop the client:
  if (!client.connected()) {
    Serial.println();
    Serial.println("disconnecting.");
    client.stop();
    // do nothing:
    while(true);
  }
}

void parseInput() {
  char hBuf[25];
  heading = strtok(&inputString[0],";");
  pitch = strtok(NULL,";");
  roll = strtok(NULL,";");
  heading.toCharArray( hBuf,24);
  turnHeading = atof(hBuf)/0.067;
  pitch.toCharArray( hBuf,24); 
  turnPitch = atof(hBuf)/0.128;
  roll.toCharArray( hBuf,24);
  turnRoll = atof(hBuf)/0.128;
  //Serial.print(turnHeading);
}


